# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from .codeable_concept import CodeableConcept
from .extension import Extension
from .period import Period
from .reference import Reference

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class DocumentReferenceContext(pydantic.BaseModel):
    """
    A reference to a document of any kind for any purpose. Provides metadata about the document so that the document can be discovered and managed. The scope of a document is any seralized object with a mime-type, so includes formal patient centric documents (CDA), cliical notes, scanned paper, and non-patient specific documents like policy text.
    """

    id: typing.Optional[str] = pydantic.Field(
        default=None,
        description="Unique id for the element within a resource (for internal references). This may be any string value that does not contain spaces.",
    )
    extension: typing.Optional[typing.List[Extension]] = pydantic.Field(
        default=None,
        description="May be used to represent additional information that is not part of the basic definition of the element. To make the use of extensions safe and manageable, there is a strict set of governance applied to the definition and use of extensions. Though any implementer can define an extension, there is a set of requirements that SHALL be met as part of the definition of the extension.",
    )
    modifier_extension: typing.Optional[typing.List[Extension]] = pydantic.Field(
        alias="modifierExtension",
        default=None,
        description="May be used to represent additional information that is not part of the basic definition of the element and that modifies the understanding of the element in which it is contained and/or the understanding of the containing element's descendants. Usually modifier elements provide negation or qualification. To make the use of extensions safe and manageable, there is a strict set of governance applied to the definition and use of extensions. Though any implementer can define an extension, there is a set of requirements that SHALL be met as part of the definition of the extension. Applications processing a resource are required to check for modifier extensions. Modifier extensions SHALL NOT change the meaning of any elements on Resource or DomainResource (including cannot change the meaning of modifierExtension itself).",
    )
    encounter: typing.Optional[typing.List[Reference]] = pydantic.Field(
        default=None,
        description="Describes the clinical encounter or type of care that the document content is associated with.",
    )
    event: typing.Optional[typing.List[CodeableConcept]] = pydantic.Field(
        default=None,
        description='This list of codes represents the main clinical acts, such as a colonoscopy or an appendectomy, being documented. In some cases, the event is inherent in the type Code, such as a "History and Physical Report" in which the procedure being documented is necessarily a "History and Physical" act.',
    )
    period: typing.Optional[Period] = pydantic.Field(
        default=None,
        description="The time period over which the service that is described by the document was provided.",
    )
    facility_type: typing.Optional[CodeableConcept] = pydantic.Field(
        alias="facilityType", default=None, description="The kind of facility where the patient was seen."
    )
    practice_setting: typing.Optional[CodeableConcept] = pydantic.Field(
        alias="practiceSetting",
        default=None,
        description="This property may convey specifics about the practice setting where the content was created, often reflecting the clinical specialty.",
    )
    source_patient_info: typing.Optional[Reference] = pydantic.Field(
        alias="sourcePatientInfo",
        default=None,
        description="The Patient Information as known when the document was published. May be a reference to a version specific, or contained.",
    )
    related: typing.Optional[typing.List[Reference]] = pydantic.Field(
        default=None, description="Related identifiers or resources associated with the DocumentReference."
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}
